@@ -15,7 +15,7 @@
 ** Random numbers are used by some of the database backends in order
 ** to generate random integer keys for tables or random filenames.
 **
-** $Id: random.c,v 1.22 2004/02/14 19:14:50 matt Exp $
+** $Id: random.c,v 1.23 2004/07/21 20:50:44 matt Exp $
 */
 #include "sqliteInt.h"
 #include "os.h"
@@ -35,15 +35,15 @@
 ** on RC4, which we know works very well.
 */
 static int randomByte(){
-  int t;
+  unsigned char t;
 
   /* All threads share a single random number generator.
   ** This structure is the current state of the generator.
   */
   static struct {
-    int isInit;          /* True if initialized */
-    int i, j;            /* State variables */
-    int s[256];          /* State variables */
+    unsigned char isInit;          /* True if initialized */
+    unsigned char i, j;            /* State variables */
+    unsigned char s[256];          /* State variables */
   } prng;
 
   /* Initialize the state of the random number generator once,
@@ -60,13 +60,12 @@ static int randomByte(){
     char k[256];
     prng.j = 0;
     prng.i = 0;
-    sqliteOsRandomSeed(k);
+    sqlite3OsRandomSeed(k);
     for(i=0; i<256; i++){
       prng.s[i] = i;
     }
     for(i=0; i<256; i++){
-      int t;
-      prng.j = (prng.j + prng.s[i] + k[i]) & 0xff;
+      prng.j += prng.s[i] + k[i];
       t = prng.s[prng.j];
       prng.s[prng.j] = prng.s[i];
       prng.s[i] = t;
@@ -76,38 +75,26 @@ static int randomByte(){
 
   /* Generate and return single random byte
   */
-  prng.i = (prng.i + 1) & 0xff;
-  prng.j = (prng.j + prng.s[prng.i]) & 0xff;
+  prng.i++;
   t = prng.s[prng.i];
+  prng.j += t;
   prng.s[prng.i] = prng.s[prng.j];
   prng.s[prng.j] = t;
-  t = prng.s[prng.i] + prng.s[prng.j];
-  return prng.s[t & 0xff];
+  t += prng.s[prng.i];
+  return prng.s[t];
 }
 
 /*
-** Return an random 8-bit integer.
+** Return N random bytes.
 */
-int sqliteRandomByte(){
-  int r;
-  sqliteOsEnterMutex();
-  r = randomByte();
-  sqliteOsLeaveMutex();
-  return r;
-}
-
-/*
-** Return a random 32-bit integer.  The integer is generated by making
-** 4 calls to sqliteRandomByte().
-*/
-int sqliteRandomInteger(){
-  int r;
-  int i;
-  sqliteOsEnterMutex();
-  r = randomByte();
-  for(i=1; i<4; i++){
-    r = (r<<8) + randomByte();
+void sqlite3Randomness(int N, void *pBuf){
+  unsigned char *zBuf = pBuf;
+  sqlite3OsEnterMutex();
+  while( N-- ){
+    *(zBuf++) = randomByte();
   }
-  sqliteOsLeaveMutex();
-  return r;
+  sqlite3OsLeaveMutex();
 }
+
+
+
