@@ -14,32 +14,49 @@
 ** This file contains functions for allocating memory, comparing
 ** strings, and stuff like that.
 **
-** $Id: util.c,v 1.22 2004/02/14 19:14:50 matt Exp $
+** $Id: util.c,v 1.23 2004/07/21 20:50:45 matt Exp $
 */
 #include "sqliteInt.h"
 #include <stdarg.h>
 #include <ctype.h>
 
+#if SQLITE_DEBUG>2 && defined(__GLIBC__)
+#include <execinfo.h>
+void print_stack_trace(){
+  void *bt[30];
+  int i;
+  int n = backtrace(bt, 30);
+
+  fprintf(stderr, "STACK: ");
+  for(i=0; i<n;i++){
+    fprintf(stderr, "%p ", bt[i]);
+  }
+  fprintf(stderr, "\n");
+}
+#else
+#define print_stack_trace()
+#endif
+
 /*
 ** If malloc() ever fails, this global variable gets set to 1.
 ** This causes the library to abort and never again function.
 */
-int sqlite_malloc_failed = 0;
+int sqlite3_malloc_failed = 0;
 
 /*
-** If MEMORY_DEBUG is defined, then use versions of malloc() and
+** If SQLITE_DEBUG is defined, then use versions of malloc() and
 ** free() that track memory usage and check for buffer overruns.
 */
-#ifdef MEMORY_DEBUG
+#ifdef SQLITE_DEBUG
 
 /*
 ** For keeping track of the number of mallocs and frees.   This
 ** is used to check for memory leaks.
 */
-int sqlite_nMalloc;         /* Number of sqliteMalloc() calls */
-int sqlite_nFree;           /* Number of sqliteFree() calls */
-int sqlite_iMallocFail;     /* Fail sqliteMalloc() after this many calls */
-#if MEMORY_DEBUG>1
+int sqlite3_nMalloc;         /* Number of sqliteMalloc() calls */
+int sqlite3_nFree;           /* Number of sqliteFree() calls */
+int sqlite3_iMallocFail;     /* Fail sqliteMalloc() after this many calls */
+#if SQLITE_DEBUG>1
 static int memcnt = 0;
 #endif
 
@@ -52,19 +69,19 @@ static int memcnt = 0;
 ** Allocate new memory and set it to zero.  Return NULL if
 ** no memory is available.
 */
-void *sqliteMalloc_(int n, int bZero, char *zFile, int line){
+void *sqlite3Malloc_(int n, int bZero, char *zFile, int line){
   void *p;
   int *pi;
   int i, k;
-  if( sqlite_iMallocFail>=0 ){
-    sqlite_iMallocFail--;
-    if( sqlite_iMallocFail==0 ){
-      sqlite_malloc_failed++;
-#if MEMORY_DEBUG>1
+  if( sqlite3_iMallocFail>=0 ){
+    sqlite3_iMallocFail--;
+    if( sqlite3_iMallocFail==0 ){
+      sqlite3_malloc_failed++;
+#if SQLITE_DEBUG>1
       fprintf(stderr,"**** failed to allocate %d bytes at %s:%d\n",
               n, zFile,line);
 #endif
-      sqlite_iMallocFail--;
+      sqlite3_iMallocFail--;
       return 0;
     }
   }
@@ -72,16 +89,17 @@ void *sqliteMalloc_(int n, int bZero, char *zFile, int line){
   k = (n+sizeof(int)-1)/sizeof(int);
   pi = malloc( (N_GUARD*2+1+k)*sizeof(int));
   if( pi==0 ){
-    sqlite_malloc_failed++;
+    sqlite3_malloc_failed++;
     return 0;
   }
-  sqlite_nMalloc++;
+  sqlite3_nMalloc++;
   for(i=0; i<N_GUARD; i++) pi[i] = 0xdead1122;
   pi[N_GUARD] = n;
   for(i=0; i<N_GUARD; i++) pi[k+1+N_GUARD+i] = 0xdead3344;
   p = &pi[N_GUARD+1];
   memset(p, bZero==0, n);
-#if MEMORY_DEBUG>1
+#if SQLITE_DEBUG>1
+  print_stack_trace();
   fprintf(stderr,"%06d malloc %d bytes at 0x%x from %s:%d\n",
       ++memcnt, n, (int)p, zFile,line);
 #endif
@@ -95,7 +113,7 @@ void *sqliteMalloc_(int n, int bZero, char *zFile, int line){
 **
 ** This routine is used for testing purposes only.
 */
-void sqliteCheckMemory(void *p, int N){
+void sqlite3CheckMemory(void *p, int N){
   int *pi = p;
   int n, i, k;
   pi -= N_GUARD+1;
@@ -113,12 +131,12 @@ void sqliteCheckMemory(void *p, int N){
 /*
 ** Free memory previously obtained from sqliteMalloc()
 */
-void sqliteFree_(void *p, char *zFile, int line){
+void sqlite3Free_(void *p, char *zFile, int line){
   if( p ){
     int *pi, i, k, n;
     pi = p;
     pi -= N_GUARD+1;
-    sqlite_nFree++;
+    sqlite3_nFree++;
     for(i=0; i<N_GUARD; i++){
       if( pi[i]!=0xdead1122 ){
         fprintf(stderr,"Low-end memory corruption at 0x%x\n", (int)p);
@@ -134,7 +152,7 @@ void sqliteFree_(void *p, char *zFile, int line){
       }
     }
     memset(pi, 0xff, (k+N_GUARD*2+1)*sizeof(int));
-#if MEMORY_DEBUG>1
+#if SQLITE_DEBUG>1
     fprintf(stderr,"%06d free %d bytes at 0x%x from %s:%d\n",
          ++memcnt, n, (int)p, zFile,line);
 #endif
@@ -147,14 +165,14 @@ void sqliteFree_(void *p, char *zFile, int line){
 ** works just like sqliteMalloc().  If n==0, then this routine
 ** works just like sqliteFree().
 */
-void *sqliteRealloc_(void *oldP, int n, char *zFile, int line){
+void *sqlite3Realloc_(void *oldP, int n, char *zFile, int line){
   int *oldPi, *pi, i, k, oldN, oldK;
   void *p;
   if( oldP==0 ){
-    return sqliteMalloc_(n,1,zFile,line);
+    return sqlite3Malloc_(n,1,zFile,line);
   }
   if( n==0 ){
-    sqliteFree_(oldP,zFile,line);
+    sqlite3Free_(oldP,zFile,line);
     return 0;
   }
   oldPi = oldP;
@@ -175,7 +193,7 @@ void *sqliteRealloc_(void *oldP, int n, char *zFile, int line){
   k = (n + sizeof(int) - 1)/sizeof(int);
   pi = malloc( (k+N_GUARD*2+1)*sizeof(int) );
   if( pi==0 ){
-    sqlite_malloc_failed++;
+    sqlite3_malloc_failed++;
     return 0;
   }
   for(i=0; i<N_GUARD; i++) pi[i] = 0xdead1122;
@@ -184,11 +202,12 @@ void *sqliteRealloc_(void *oldP, int n, char *zFile, int line){
   p = &pi[N_GUARD+1];
   memcpy(p, oldP, n>oldN ? oldN : n);
   if( n>oldN ){
-    memset(&((char*)p)[oldN], 0, n-oldN);
+    memset(&((char*)p)[oldN], 0x55, n-oldN);
   }
   memset(oldPi, 0xab, (oldK+N_GUARD+2)*sizeof(int));
   free(oldPi);
-#if MEMORY_DEBUG>1
+#if SQLITE_DEBUG>1
+  print_stack_trace();
   fprintf(stderr,"%06d realloc %d to %d bytes at 0x%x to 0x%x at %s:%d\n",
     ++memcnt, oldN, n, (int)oldP, (int)p, zFile, line);
 #endif
@@ -203,12 +222,12 @@ void *sqliteRealloc_(void *oldP, int n, char *zFile, int line){
 ** the SQLite library.  That way clients can free the string using free()
 ** rather than having to call sqliteFree().
 */
-void sqliteStrRealloc(char **pz){
+void sqlite3StrRealloc(char **pz){
   char *zNew;
   if( pz==0 || *pz==0 ) return;
   zNew = malloc( strlen(*pz) + 1 );
   if( zNew==0 ){
-    sqlite_malloc_failed++;
+    sqlite3_malloc_failed++;
     sqliteFree(*pz);
     *pz = 0;
   }
@@ -220,39 +239,46 @@ void sqliteStrRealloc(char **pz){
 /*
 ** Make a copy of a string in memory obtained from sqliteMalloc()
 */
-char *sqliteStrDup_(const char *z, char *zFile, int line){
+char *sqlite3StrDup_(const char *z, char *zFile, int line){
   char *zNew;
   if( z==0 ) return 0;
-  zNew = sqliteMalloc_(strlen(z)+1, 0, zFile, line);
+  zNew = sqlite3Malloc_(strlen(z)+1, 0, zFile, line);
   if( zNew ) strcpy(zNew, z);
   return zNew;
 }
-char *sqliteStrNDup_(const char *z, int n, char *zFile, int line){
+char *sqlite3StrNDup_(const char *z, int n, char *zFile, int line){
   char *zNew;
   if( z==0 ) return 0;
-  zNew = sqliteMalloc_(n+1, 0, zFile, line);
+  zNew = sqlite3Malloc_(n+1, 0, zFile, line);
   if( zNew ){
     memcpy(zNew, z, n);
     zNew[n] = 0;
   }
   return zNew;
 }
-#endif /* MEMORY_DEBUG */
+
+/*
+** A version of sqliteFree that is always a function, not a macro.
+*/
+void sqlite3FreeX(void *p){
+  sqliteFree(p);
+}
+#endif /* SQLITE_DEBUG */
 
 /*
 ** The following versions of malloc() and free() are for use in a
 ** normal build.
 */
-#if !defined(MEMORY_DEBUG)
+#if !defined(SQLITE_DEBUG)
 
 /*
 ** Allocate new memory and set it to zero.  Return NULL if
 ** no memory is available.  See also sqliteMallocRaw().
 */
-void *sqliteMalloc(int n){
+void *sqlite3Malloc(int n){
   void *p;
   if( (p = malloc(n))==0 ){
-    if( n>0 ) sqlite_malloc_failed++;
+    if( n>0 ) sqlite3_malloc_failed++;
   }else{
     memset(p, 0, n);
   }
@@ -263,10 +289,10 @@ void *sqliteMalloc(int n){
 ** Allocate new memory but do not set it to zero.  Return NULL if
 ** no memory is available.  See also sqliteMalloc().
 */
-void *sqliteMallocRaw(int n){
+void *sqlite3MallocRaw(int n){
   void *p;
   if( (p = malloc(n))==0 ){
-    if( n>0 ) sqlite_malloc_failed++;
+    if( n>0 ) sqlite3_malloc_failed++;
   }
   return p;
 }
@@ -274,7 +300,7 @@ void *sqliteMallocRaw(int n){
 /*
 ** Free memory previously obtained from sqliteMalloc()
 */
-void sqliteFree(void *p){
+void sqlite3FreeX(void *p){
   if( p ){
     free(p);
   }
@@ -285,7 +311,7 @@ void sqliteFree(void *p){
 ** works just like sqliteMalloc().  If n==0, then this routine
 ** works just like sqliteFree().
 */
-void *sqliteRealloc(void *p, int n){
+void *sqlite3Realloc(void *p, int n){
   void *p2;
   if( p==0 ){
     return sqliteMalloc(n);
@@ -296,7 +322,7 @@ void *sqliteRealloc(void *p, int n){
   }
   p2 = realloc(p, n);
   if( p2==0 ){
-    sqlite_malloc_failed++;
+    sqlite3_malloc_failed++;
   }
   return p2;
 }
@@ -304,14 +330,14 @@ void *sqliteRealloc(void *p, int n){
 /*
 ** Make a copy of a string in memory obtained from sqliteMalloc()
 */
-char *sqliteStrDup(const char *z){
+char *sqlite3StrDup(const char *z){
   char *zNew;
   if( z==0 ) return 0;
   zNew = sqliteMallocRaw(strlen(z)+1);
   if( zNew ) strcpy(zNew, z);
   return zNew;
 }
-char *sqliteStrNDup(const char *z, int n){
+char *sqlite3StrNDup(const char *z, int n){
   char *zNew;
   if( z==0 ) return 0;
   zNew = sqliteMallocRaw(n+1);
@@ -321,7 +347,7 @@ char *sqliteStrNDup(const char *z, int n){
   }
   return zNew;
 }
-#endif /* !defined(MEMORY_DEBUG) */
+#endif /* !defined(SQLITE_DEBUG) */
 
 /*
 ** Create a string from the 2nd and subsequent arguments (up to the
@@ -330,7 +356,7 @@ char *sqliteStrNDup(const char *z, int n){
 ** point to that string.  The 1st argument must either be NULL or 
 ** point to memory obtained from sqliteMalloc().
 */
-void sqliteSetString(char **pz, const char *zFirst, ...){
+void sqlite3SetString(char **pz, const char *zFirst, ...){
   va_list ap;
   int nByte;
   const char *z;
@@ -356,21 +382,21 @@ void sqliteSetString(char **pz, const char *zFirst, ...){
     zResult += strlen(zResult);
   }
   va_end(ap);
-#ifdef MEMORY_DEBUG
-#if MEMORY_DEBUG>1
+#ifdef SQLITE_DEBUG
+#if SQLITE_DEBUG>1
   fprintf(stderr,"string at 0x%x is %s\n", (int)*pz, *pz);
 #endif
 #endif
 }
 
 /*
-** Works like sqliteSetString, but each string is now followed by
+** Works like sqlite3SetString, but each string is now followed by
 ** a length integer which specifies how much of the source string 
 ** to copy (in bytes).  -1 means use the whole string.  The 1st 
 ** argument must either be NULL or point to memory obtained from 
 ** sqliteMalloc().
 */
-void sqliteSetNString(char **pz, ...){
+void sqlite3SetNString(char **pz, ...){
   va_list ap;
   int nByte;
   const char *z;
@@ -393,18 +419,55 @@ void sqliteSetNString(char **pz, ...){
   while( (z = va_arg(ap, const char*))!=0 ){
     n = va_arg(ap, int);
     if( n<=0 ) n = strlen(z);
-    strncpy(zResult, z, n);
+    memcpy(zResult, z, n);
     zResult += n;
   }
   *zResult = 0;
-#ifdef MEMORY_DEBUG
-#if MEMORY_DEBUG>1
+#ifdef SQLITE_DEBUG
+#if SQLITE_DEBUG>1
   fprintf(stderr,"string at 0x%x is %s\n", (int)*pz, *pz);
 #endif
 #endif
   va_end(ap);
 }
 
+/*
+** Set the most recent error code and error string for the sqlite
+** handle "db". The error code is set to "err_code".
+**
+** If it is not NULL, string zFormat specifies the format of the
+** error string in the style of the printf functions: The following
+** format characters are allowed:
+**
+**      %s      Insert a string
+**      %z      A string that should be freed after use
+**      %d      Insert an integer
+**      %T      Insert a token
+**      %S      Insert the first element of a SrcList
+**
+** zFormat and any string tokens that follow it are assumed to be
+** encoded in UTF-8.
+**
+** To clear the most recent error for slqite handle "db", sqlite3Error
+** should be called with err_code set to SQLITE_OK and zFormat set
+** to NULL.
+*/
+void sqlite3Error(sqlite *db, int err_code, const char *zFormat, ...){
+  if( db && (db->pErr || (db->pErr = sqlite3ValueNew())) ){
+    db->errCode = err_code;
+    if( zFormat ){
+      char *z;
+      va_list ap;
+      va_start(ap, zFormat);
+      z = sqlite3VMPrintf(zFormat, ap);
+      va_end(ap);
+      sqlite3ValueSetStr(db->pErr, -1, z, SQLITE_UTF8, sqlite3FreeX);
+    }else{
+      sqlite3ValueSetStr(db->pErr, 0, 0, SQLITE_UTF8, SQLITE_STATIC);
+    }
+  }
+}
+
 /*
 ** Add an error message to pParse->zErrMsg and increment pParse->nErr.
 ** The following formatting characters are allowed:
@@ -414,123 +477,21 @@ void sqliteSetNString(char **pz, ...){
 **      %d      Insert an integer
 **      %T      Insert a token
 **      %S      Insert the first element of a SrcList
+**
+** This function should be used to report any error that occurs whilst
+** compiling an SQL statement (i.e. within sqlite3_prepare()). The
+** last thing the sqlite3_prepare() function does is copy the error
+** stored by this function into the database handle using sqlite3Error().
+** Function sqlite3Error() should be used during statement execution
+** (sqlite3_step() etc.).
 */
-void sqliteErrorMsg(Parse *pParse, const char *zFormat, ...){
+void sqlite3ErrorMsg(Parse *pParse, const char *zFormat, ...){
   va_list ap;
-  int nByte;
-  int i, j;
-  char *z;
-  static char zNull[] = "NULL";
-
   pParse->nErr++;
-  nByte = 1 + strlen(zFormat);
-  va_start(ap, zFormat);
-  for(i=0; zFormat[i]; i++){
-    if( zFormat[i]!='%' || zFormat[i+1]==0 ) continue;
-    i++;
-    switch( zFormat[i] ){
-      case 'd': {
-        (void)va_arg(ap, int);
-        nByte += 20;
-        break;
-      }
-      case 'z':
-      case 's': {
-        char *z2 = va_arg(ap, char*);
-        if( z2==0 ) z2 = zNull;
-        nByte += strlen(z2);
-        break;
-      }
-      case 'T': {
-        Token *p = va_arg(ap, Token*);
-        nByte += p->n;
-        break;
-      }
-      case 'S': {
-        SrcList *p = va_arg(ap, SrcList*);
-        int k = va_arg(ap, int);
-        assert( p->nSrc>k && k>=0 );
-        nByte += strlen(p->a[k].zName);
-        if( p->a[k].zDatabase && p->a[k].zDatabase[0] ){
-          nByte += strlen(p->a[k].zDatabase)+1;
-        }
-        break;
-      }
-      default: {
-        nByte++;
-        break;
-      }
-    }
-  }
-  va_end(ap);
-  z = sqliteMalloc( nByte );
-  if( z==0 ) return;
   sqliteFree(pParse->zErrMsg);
-  pParse->zErrMsg = z;
   va_start(ap, zFormat);
-  for(i=j=0; zFormat[i]; i++){
-    if( zFormat[i]!='%' || zFormat[i+1]==0 ) continue;
-    if( i>j ){
-      memcpy(z, &zFormat[j], i-j);
-      z += i-j;
-    }
-    j = i+2;
-    i++;
-    switch( zFormat[i] ){
-      case 'd': {
-        int x = va_arg(ap, int);
-        sprintf(z, "%d", x);
-        z += strlen(z);
-        break;
-      }
-      case 'z':
-      case 's': {
-        int len;
-        char *z2 = va_arg(ap, char*);
-        if( z2==0 ) z2 = zNull;
-        len = strlen(z2);
-        memcpy(z, z2, len);
-        z += len;
-        if( zFormat[i]=='z' && z2!=zNull ){
-          sqliteFree(z2);
-        }
-        break;
-      }
-      case 'T': {
-        Token *p = va_arg(ap, Token*);
-        memcpy(z, p->z, p->n);
-        z += p->n;
-        break;
-      }
-      case 'S': {
-        int len;
-        SrcList *p = va_arg(ap, SrcList*);
-        int k = va_arg(ap, int);
-        assert( p->nSrc>k && k>=0 );
-        if( p->a[k].zDatabase && p->a[k].zDatabase[0] ){
-          len = strlen(p->a[k].zDatabase);
-          memcpy(z, p->a[k].zDatabase, len);
-          z += len;
-          *(z++) = '.';
-        }
-        len = strlen(p->a[k].zName);
-        memcpy(z, p->a[k].zName, len);
-        z += len;
-        break;
-      }
-      default: {
-        *(z++) = zFormat[i];
-        break;
-      }
-    }
-  }
+  pParse->zErrMsg = sqlite3VMPrintf(zFormat, ap);
   va_end(ap);
-  if( i>j ){
-    memcpy(z, &zFormat[j], i-j);
-    z += i-j;
-  }
-  assert( (z - pParse->zErrMsg) < nByte );
-  *z = 0;
 }
 
 /*
@@ -543,7 +504,7 @@ void sqliteErrorMsg(Parse *pParse, const char *zFormat, ...){
 ** brackets from around identifers.  For example:  "[a-b-c]" becomes
 ** "a-b-c".
 */
-void sqliteDequote(char *z){
+void sqlite3Dequote(char *z){
   int quote;
   int i, j;
   if( z==0 ) return;
@@ -594,7 +555,7 @@ static unsigned char UpperToLower[] = {
 ** This function computes a hash on the name of a keyword.
 ** Case is not significant.
 */
-int sqliteHashNoCase(const char *z, int n){
+int sqlite3HashNoCase(const char *z, int n){
   int h = 0;
   if( n<=0 ) n = strlen(z);
   while( n > 0  ){
@@ -608,44 +569,51 @@ int sqliteHashNoCase(const char *z, int n){
 ** Some systems have stricmp().  Others have strcasecmp().  Because
 ** there is no consistency, we will define our own.
 */
-int sqliteStrICmp(const char *zLeft, const char *zRight){
+int sqlite3StrICmp(const char *zLeft, const char *zRight){
   register unsigned char *a, *b;
   a = (unsigned char *)zLeft;
   b = (unsigned char *)zRight;
   while( *a!=0 && UpperToLower[*a]==UpperToLower[*b]){ a++; b++; }
   return *a - *b;
 }
-int sqliteStrNICmp(const char *zLeft, const char *zRight, int N){
+int sqlite3StrNICmp(const char *zLeft, const char *zRight, int N){
   register unsigned char *a, *b;
   a = (unsigned char *)zLeft;
   b = (unsigned char *)zRight;
   while( N-- > 0 && *a!=0 && UpperToLower[*a]==UpperToLower[*b]){ a++; b++; }
-  return N<0 ? 0 : *a - *b;
+  return N<0 ? 0 : UpperToLower[*a] - UpperToLower[*b];
 }
 
 /*
 ** Return TRUE if z is a pure numeric string.  Return FALSE if the
-** string contains any character which is not part of a number.
+** string contains any character which is not part of a number. If
+** the string is numeric and contains the '.' character, set *realnum
+** to TRUE (otherwise FALSE).
 **
 ** Am empty string is considered non-numeric.
 */
-int sqliteIsNumber(const char *z){
-  if( *z=='-' || *z=='+' ) z++;
+int sqlite3IsNumber(const char *z, int *realnum, u8 enc){
+  int incr = (enc==SQLITE_UTF8?1:2);
+  if( enc==SQLITE_UTF16BE ) z++;
+  if( *z=='-' || *z=='+' ) z += incr;
   if( !isdigit(*z) ){
     return 0;
   }
-  z++;
-  while( isdigit(*z) ){ z++; }
+  z += incr;
+  if( realnum ) *realnum = 0;
+  while( isdigit(*z) ){ z += incr; }
   if( *z=='.' ){
-    z++;
+    z += incr;
     if( !isdigit(*z) ) return 0;
-    while( isdigit(*z) ){ z++; }
+    while( isdigit(*z) ){ z += incr; }
+    if( realnum ) *realnum = 1;
   }
   if( *z=='e' || *z=='E' ){
-    z++;
-    if( *z=='+' || *z=='-' ) z++;
+    z += incr;
+    if( *z=='+' || *z=='-' ) z += incr;
     if( !isdigit(*z) ) return 0;
-    while( isdigit(*z) ){ z++; }
+    while( isdigit(*z) ){ z += incr; }
+    if( realnum ) *realnum = 1;
   }
   return *z==0;
 }
@@ -662,7 +630,7 @@ int sqliteIsNumber(const char *z){
 ** of "." depending on how locale is set.  But that would cause problems
 ** for SQL.  So this routine always uses "." regardless of locale.
 */
-double sqliteAtoF(const char *z){
+double sqlite3AtoF(const char *z, const char **pzEnd){
   int sign = 1;
   LONGDOUBLE_TYPE v1 = 0.0;
   if( *z=='-' ){
@@ -710,9 +678,42 @@ double sqliteAtoF(const char *z){
       v1 *= scale;
     }
   }
+  if( pzEnd ) *pzEnd = z;
   return sign<0 ? -v1 : v1;
 }
 
+/*
+** Return TRUE if zNum is a 64-bit signed integer and write
+** the value of the integer into *pNum.  If zNum is not an integer
+** or is an integer that is too large to be expressed with 64 bits,
+** then return false.  If n>0 and the integer is string is not
+** exactly n bytes long, return false.
+**
+** When this routine was originally written it dealt with only
+** 32-bit numbers.  At that time, it was much faster than the
+** atoi() library routine in RedHat 7.2.
+*/
+int sqlite3atoi64(const char *zNum, i64 *pNum){
+  i64 v = 0;
+  int neg;
+  int i, c;
+  if( *zNum=='-' ){
+    neg = 1;
+    zNum++;
+  }else if( *zNum=='+' ){
+    neg = 0;
+    zNum++;
+  }else{
+    neg = 0;
+  }
+  for(i=0; (c=zNum[i])>='0' && c<='9'; i++){
+    v = v*10 + c - '0';
+  }
+  *pNum = neg ? -v : v;
+  return c==0 && i>0 && 
+      (i<19 || (i==19 && memcmp(zNum,"9223372036854775807",19)<=0));
+}
+
 /*
 ** The string zNum represents an integer.  There might be some other
 ** information following the integer too, but that part is ignored.
@@ -724,254 +725,51 @@ double sqliteAtoF(const char *z){
 ** 2147483648 will not fit in 32 bits.  So it seems safer to return
 ** false.
 */
-int sqliteFitsIn32Bits(const char *zNum){
+static int sqlite3FitsIn32Bits(const char *zNum){
   int i, c;
   if( *zNum=='-' || *zNum=='+' ) zNum++;
   for(i=0; (c=zNum[i])>='0' && c<='9'; i++){}
   return i<10 || (i==10 && memcmp(zNum,"2147483647",10)<=0);
 }
 
-/* This comparison routine is what we use for comparison operations
-** between numeric values in an SQL expression.  "Numeric" is a little
-** bit misleading here.  What we mean is that the strings have a
-** type of "numeric" from the point of view of SQL.  The strings
-** do not necessarily contain numbers.  They could contain text.
-**
-** If the input strings both look like actual numbers then they
-** compare in numerical order.  Numerical strings are always less 
-** than non-numeric strings so if one input string looks like a
-** number and the other does not, then the one that looks like
-** a number is the smaller.  Non-numeric strings compare in 
-** lexigraphical order (the same order as strcmp()).
+/*
+** If zNum represents an integer that will fit in 32-bits, then set
+** *pValue to that integer and return true.  Otherwise return false.
 */
-int sqliteCompare(const char *atext, const char *btext){
-  int result;
-  int isNumA, isNumB;
-  if( atext==0 ){
-    return -1;
-  }else if( btext==0 ){
+int sqlite3GetInt32(const char *zNum, int *pValue){
+  if( sqlite3FitsIn32Bits(zNum) ){
+    *pValue = atoi(zNum);
     return 1;
   }
-  isNumA = sqliteIsNumber(atext);
-  isNumB = sqliteIsNumber(btext);
-  if( isNumA ){
-    if( !isNumB ){
-      result = -1;
-    }else{
-      double rA, rB;
-      rA = sqliteAtoF(atext);
-      rB = sqliteAtoF(btext);
-      if( rA<rB ){
-        result = -1;
-      }else if( rA>rB ){
-        result = +1;
-      }else{
-        result = 0;
-      }
-    }
-  }else if( isNumB ){
-    result = +1;
-  }else {
-    result = strcmp(atext, btext);
-  }
-  return result; 
+  return 0;
 }
 
 /*
-** This routine is used for sorting.  Each key is a list of one or more
-** null-terminated elements.  The list is terminated by two nulls in
-** a row.  For example, the following text is a key with three elements
+** The string zNum represents an integer.  There might be some other
+** information following the integer too, but that part is ignored.
+** If the integer that the prefix of zNum represents will fit in a
+** 64-bit signed integer, return TRUE.  Otherwise return FALSE.
 **
-**            Aone\000Dtwo\000Athree\000\000
-**
-** All elements begin with one of the characters "+-AD" and end with "\000"
-** with zero or more text elements in between.  Except, NULL elements
-** consist of the special two-character sequence "N\000".
-**
-** Both arguments will have the same number of elements.  This routine
-** returns negative, zero, or positive if the first argument is less
-** than, equal to, or greater than the first.  (Result is a-b).
-**
-** Each element begins with one of the characters "+", "-", "A", "D".
-** This character determines the sort order and collating sequence:
-**
-**     +      Sort numerically in ascending order
-**     -      Sort numerically in descending order
-**     A      Sort as strings in ascending order
-**     D      Sort as strings in descending order.
-**
-** For the "+" and "-" sorting, pure numeric strings (strings for which the
-** isNum() function above returns TRUE) always compare less than strings
-** that are not pure numerics.  Non-numeric strings compare in memcmp()
-** order.  This is the same sort order as the sqliteCompare() function
-** above generates.
-**
-** The last point is a change from version 2.6.3 to version 2.7.0.  In
-** version 2.6.3 and earlier, substrings of digits compare in numerical 
-** and case was used only to break a tie.
-**
-** Elements that begin with 'A' or 'D' compare in memcmp() order regardless
-** of whether or not they look like a number.
-**
-** Note that the sort order imposed by the rules above is the same
-** from the ordering defined by the "<", "<=", ">", and ">=" operators
-** of expressions and for indices.  This was not the case for version
-** 2.6.3 and earlier.
+** This routine returns FALSE for the string -9223372036854775808 even that
+** that number will, in theory fit in a 64-bit integer.  Positive
+** 9223373036854775808 will not fit in 64 bits.  So it seems safer to return
+** false.
 */
-int sqliteSortCompare(const char *a, const char *b){
-  int res = 0;
-  int isNumA, isNumB;
-  int dir = 0;
-
-  while( res==0 && *a && *b ){
-    if( a[0]=='N' || b[0]=='N' ){
-      if( a[0]==b[0] ){
-        a += 2;
-        b += 2;
-        continue;
-      }
-      if( a[0]=='N' ){
-        dir = b[0];
-        res = -1;
-      }else{
-        dir = a[0];
-        res = +1;
-      }
-      break;
-    }
-    assert( a[0]==b[0] );
-    if( (dir=a[0])=='A' || a[0]=='D' ){
-      res = strcmp(&a[1],&b[1]);
-      if( res ) break;
-    }else{
-      isNumA = sqliteIsNumber(&a[1]);
-      isNumB = sqliteIsNumber(&b[1]);
-      if( isNumA ){
-        double rA, rB;
-        if( !isNumB ){
-          res = -1;
-          break;
-        }
-        rA = sqliteAtoF(&a[1]);
-        rB = sqliteAtoF(&b[1]);
-        if( rA<rB ){
-          res = -1;
-          break;
-        }
-        if( rA>rB ){
-          res = +1;
-          break;
-        }
-      }else if( isNumB ){
-        res = +1;
-        break;
-      }else{
-        res = strcmp(&a[1],&b[1]);
-        if( res ) break;
-      }
-    }
-    a += strlen(&a[1]) + 2;
-    b += strlen(&b[1]) + 2;
-  }
-  if( dir=='-' || dir=='D' ) res = -res;
-  return res;
+int sqlite3FitsIn64Bits(const char *zNum){
+  int i, c;
+  if( *zNum=='-' || *zNum=='+' ) zNum++;
+  for(i=0; (c=zNum[i])>='0' && c<='9'; i++){}
+  return i<19 || (i==19 && memcmp(zNum,"9223372036854775807",19)<=0);
 }
 
-/*
-** Some powers of 64.  These constants are needed in the
-** sqliteRealToSortable() routine below.
-*/
-#define _64e3  (64.0 * 64.0 * 64.0)
-#define _64e4  (64.0 * 64.0 * 64.0 * 64.0)
-#define _64e15 (_64e3 * _64e4 * _64e4 * _64e4)
-#define _64e16 (_64e4 * _64e4 * _64e4 * _64e4)
-#define _64e63 (_64e15 * _64e16 * _64e16 * _64e16)
-#define _64e64 (_64e16 * _64e16 * _64e16 * _64e16)
-
-/*
-** The following procedure converts a double-precision floating point
-** number into a string.  The resulting string has the property that
-** two such strings comparied using strcmp() or memcmp() will give the
-** same results as a numeric comparison of the original floating point
-** numbers.
-**
-** This routine is used to generate database keys from floating point
-** numbers such that the keys sort in the same order as the original
-** floating point numbers even though the keys are compared using
-** memcmp().
-**
-** The calling function should have allocated at least 14 characters
-** of space for the buffer z[].
-*/
-void sqliteRealToSortable(double r, char *z){
-  int neg;
-  int exp;
-  int cnt = 0;
-
-  /* This array maps integers between 0 and 63 into base-64 digits.
-  ** The digits must be chosen such at their ASCII codes are increasing.
-  ** This means we can not use the traditional base-64 digit set. */
-  static const char zDigit[] = 
-     "0123456789"
-     "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-     "abcdefghijklmnopqrstuvwxyz"
-     "|~";
-  if( r<0.0 ){
-    neg = 1;
-    r = -r;
-    *z++ = '-';
-  } else {
-    neg = 0;
-    *z++ = '0';
-  }
-  exp = 0;
-
-  if( r==0.0 ){
-    exp = -1024;
-  }else if( r<(0.5/64.0) ){
-    while( r < 0.5/_64e64 && exp > -961  ){ r *= _64e64;  exp -= 64; }
-    while( r < 0.5/_64e16 && exp > -1009 ){ r *= _64e16;  exp -= 16; }
-    while( r < 0.5/_64e4  && exp > -1021 ){ r *= _64e4;   exp -= 4; }
-    while( r < 0.5/64.0   && exp > -1024 ){ r *= 64.0;    exp -= 1; }
-  }else if( r>=0.5 ){
-    while( r >= 0.5*_64e63 && exp < 960  ){ r *= 1.0/_64e64; exp += 64; }
-    while( r >= 0.5*_64e15 && exp < 1008 ){ r *= 1.0/_64e16; exp += 16; }
-    while( r >= 0.5*_64e3  && exp < 1020 ){ r *= 1.0/_64e4;  exp += 4; }
-    while( r >= 0.5        && exp < 1023 ){ r *= 1.0/64.0;   exp += 1; }
-  }
-  if( neg ){
-    exp = -exp;
-    r = -r;
-  }
-  exp += 1024;
-  r += 0.5;
-  if( exp<0 ) return;
-  if( exp>=2048 || r>=1.0 ){
-    strcpy(z, "~~~~~~~~~~~~");
-    return;
-  }
-  *z++ = zDigit[(exp>>6)&0x3f];
-  *z++ = zDigit[exp & 0x3f];
-  while( r>0.0 && cnt<10 ){
-    int digit;
-    r *= 64.0;
-    digit = (int)r;
-    assert( digit>=0 && digit<64 );
-    *z++ = zDigit[digit & 0x3f];
-    r -= digit;
-    cnt++;
-  }
-  *z = 0;
-}
-
-#ifdef SQLITE_UTF8
+#if 1  /* We are now always UTF-8 */
 /*
 ** X is a pointer to the first byte of a UTF-8 character.  Increment
 ** X so that it points to the next character.  This only works right
 ** if X points to a well-formed UTF-8 string.
 */
 #define sqliteNextChar(X)  while( (0xc0&*++(X))==0x80 ){}
-#define sqliteCharVal(X)   sqlite_utf8_to_int(X)
+#define sqliteCharVal(X)   sqlite3ReadUtf8(X)
 
 #else /* !defined(SQLITE_UTF8) */
 /*
@@ -983,13 +781,13 @@ void sqliteRealToSortable(double r, char *z){
 #endif /* defined(SQLITE_UTF8) */
 
 
-#ifdef SQLITE_UTF8
+#if 1  /* We are now always UTF-8 */
 /*
 ** Convert the UTF-8 character to which z points into a 31-bit
 ** UCS character.  This only works right if z points to a well-formed
 ** UTF-8 string.
 */
-static int sqlite_utf8_to_int(const unsigned char *z){
+int sqlite3ReadUtf8(const unsigned char *z){
   int c;
   static const int initVal[] = {
       0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
@@ -1048,7 +846,7 @@ static int sqlite_utf8_to_int(const unsigned char *z){
 **         abc[*]xyz        Matches "abc*xyz" only
 */
 int 
-sqliteGlobCompare(const unsigned char *zPattern, const unsigned char *zString){
+sqlite3GlobCompare(const unsigned char *zPattern, const unsigned char *zString){
   register int c;
   int invert;
   int seen;
@@ -1066,7 +864,7 @@ sqliteGlobCompare(const unsigned char *zPattern, const unsigned char *zString){
         }
         if( c==0 ) return 1;
         if( c=='[' ){
-          while( *zString && sqliteGlobCompare(&zPattern[1],zString)==0 ){
+          while( *zString && sqlite3GlobCompare(&zPattern[1],zString)==0 ){
             sqliteNextChar(zString);
           }
           return *zString!=0;
@@ -1074,7 +872,7 @@ sqliteGlobCompare(const unsigned char *zPattern, const unsigned char *zString){
           while( (c2 = *zString)!=0 ){
             while( c2 != 0 && c2 != c ){ c2 = *++zString; }
             if( c2==0 ) return 0;
-            if( sqliteGlobCompare(&zPattern[1],zString) ) return 1;
+            if( sqlite3GlobCompare(&zPattern[1],zString) ) return 1;
             sqliteNextChar(zString);
           }
           return 0;
@@ -1127,57 +925,6 @@ sqliteGlobCompare(const unsigned char *zPattern, const unsigned char *zString){
   return *zString==0;
 }
 
-/*
-** Compare two UTF-8 strings for equality using the "LIKE" operator of
-** SQL.  The '%' character matches any sequence of 0 or more
-** characters and '_' matches any single character.  Case is
-** not significant.
-**
-** This routine is just an adaptation of the sqliteGlobCompare()
-** routine above.
-*/
-int 
-sqliteLikeCompare(const unsigned char *zPattern, const unsigned char *zString){
-  register int c;
-  int c2;
-
-  while( (c = UpperToLower[*zPattern])!=0 ){
-    switch( c ){
-      case '%': {
-        while( (c=zPattern[1]) == '%' || c == '_' ){
-          if( c=='_' ){
-            if( *zString==0 ) return 0;
-            sqliteNextChar(zString);
-          }
-          zPattern++;
-        }
-        if( c==0 ) return 1;
-        c = UpperToLower[c];
-        while( (c2=UpperToLower[*zString])!=0 ){
-          while( c2 != 0 && c2 != c ){ c2 = UpperToLower[*++zString]; }
-          if( c2==0 ) return 0;
-          if( sqliteLikeCompare(&zPattern[1],zString) ) return 1;
-          sqliteNextChar(zString);
-        }
-        return 0;
-      }
-      case '_': {
-        if( *zString==0 ) return 0;
-        sqliteNextChar(zString);
-        zPattern++;
-        break;
-      }
-      default: {
-        if( c != UpperToLower[*zString] ) return 0;
-        zPattern++;
-        zString++;
-        break;
-      }
-    }
-  }
-  return *zString==0;
-}
-
 /*
 ** Change the sqlite.magic from SQLITE_MAGIC_OPEN to SQLITE_MAGIC_BUSY.
 ** Return an error (non-zero) if the magic was not SQLITE_MAGIC_OPEN
@@ -1193,15 +940,14 @@ sqliteLikeCompare(const unsigned char *zPattern, const unsigned char *zString){
 ** Ticket #202:  If db->magic is not a valid open value, take care not
 ** to modify the db structure at all.  It could be that db is a stale
 ** pointer.  In other words, it could be that there has been a prior
-** call to sqlite_close(db) and db has been deallocated.  And we do
+** call to sqlite3_close(db) and db has been deallocated.  And we do
 ** not want to write into deallocated memory.
 */
-int sqliteSafetyOn(sqlite *db){
+int sqlite3SafetyOn(sqlite *db){
   if( db->magic==SQLITE_MAGIC_OPEN ){
     db->magic = SQLITE_MAGIC_BUSY;
     return 0;
-  }else if( db->magic==SQLITE_MAGIC_BUSY || db->magic==SQLITE_MAGIC_ERROR
-             || db->want_to_close ){
+  }else if( db->magic==SQLITE_MAGIC_BUSY || db->magic==SQLITE_MAGIC_ERROR ){
     db->magic = SQLITE_MAGIC_ERROR;
     db->flags |= SQLITE_Interrupt;
   }
@@ -1213,12 +959,11 @@ int sqliteSafetyOn(sqlite *db){
 ** Return an error (non-zero) if the magic was not SQLITE_MAGIC_BUSY
 ** when this routine is called.
 */
-int sqliteSafetyOff(sqlite *db){
+int sqlite3SafetyOff(sqlite *db){
   if( db->magic==SQLITE_MAGIC_BUSY ){
     db->magic = SQLITE_MAGIC_OPEN;
     return 0;
-  }else if( db->magic==SQLITE_MAGIC_OPEN || db->magic==SQLITE_MAGIC_ERROR
-             || db->want_to_close ){
+  }else if( db->magic==SQLITE_MAGIC_OPEN || db->magic==SQLITE_MAGIC_ERROR ){
     db->magic = SQLITE_MAGIC_ERROR;
     db->flags |= SQLITE_Interrupt;
   }
@@ -1226,18 +971,187 @@ int sqliteSafetyOff(sqlite *db){
 }
 
 /*
-** Check to make sure we are not currently executing an sqlite_exec().
-** If we are currently in an sqlite_exec(), return true and set
+** Check to make sure we are not currently executing an sqlite3_exec().
+** If we are currently in an sqlite3_exec(), return true and set
 ** sqlite.magic to SQLITE_MAGIC_ERROR.  This will cause a complete
 ** shutdown of the database.
 **
 ** This routine is used to try to detect when API routines are called
 ** at the wrong time or in the wrong sequence.
 */
-int sqliteSafetyCheck(sqlite *db){
+int sqlite3SafetyCheck(sqlite *db){
   if( db->pVdbe!=0 ){
     db->magic = SQLITE_MAGIC_ERROR;
     return 1;
   }
   return 0;
 }
+
+/*
+** The variable-length integer encoding is as follows:
+**
+** KEY:
+**         A = 0xxxxxxx    7 bits of data and one flag bit
+**         B = 1xxxxxxx    7 bits of data and one flag bit
+**         C = xxxxxxxx    8 bits of data
+**
+**  7 bits - A
+** 14 bits - BA
+** 21 bits - BBA
+** 28 bits - BBBA
+** 35 bits - BBBBA
+** 42 bits - BBBBBA
+** 49 bits - BBBBBBA
+** 56 bits - BBBBBBBA
+** 64 bits - BBBBBBBBC
+*/
+
+/*
+** Write a 64-bit variable-length integer to memory starting at p[0].
+** The length of data write will be between 1 and 9 bytes.  The number
+** of bytes written is returned.
+**
+** A variable-length integer consists of the lower 7 bits of each byte
+** for all bytes that have the 8th bit set and one byte with the 8th
+** bit clear.  Except, if we get to the 9th byte, it stores the full
+** 8 bits and is the last byte.
+*/
+int sqlite3PutVarint(unsigned char *p, u64 v){
+  int i, j, n;
+  u8 buf[10];
+  if( v & 0xff00000000000000 ){
+    p[8] = v;
+    v >>= 8;
+    for(i=7; i>=0; i--){
+      p[i] = (v & 0x7f) | 0x80;
+      v >>= 7;
+    }
+    return 9;
+  }    
+  n = 0;
+  do{
+    buf[n++] = (v & 0x7f) | 0x80;
+    v >>= 7;
+  }while( v!=0 );
+  buf[0] &= 0x7f;
+  assert( n<=9 );
+  for(i=0, j=n-1; j>=0; j--, i++){
+    p[i] = buf[j];
+  }
+  return n;
+}
+
+/*
+** Read a 64-bit variable-length integer from memory starting at p[0].
+** Return the number of bytes read.  The value is stored in *v.
+*/
+int sqlite3GetVarint(const unsigned char *p, u64 *v){
+  u32 x;
+  u64 x64;
+  int n;
+  unsigned char c;
+  if( ((c = p[0]) & 0x80)==0 ){
+    *v = c;
+    return 1;
+  }
+  x = c & 0x7f;
+  if( ((c = p[1]) & 0x80)==0 ){
+    *v = (x<<7) | c;
+    return 2;
+  }
+  x = (x<<7) | (c&0x7f);
+  if( ((c = p[2]) & 0x80)==0 ){
+    *v = (x<<7) | c;
+    return 3;
+  }
+  x = (x<<7) | (c&0x7f);
+  if( ((c = p[3]) & 0x80)==0 ){
+    *v = (x<<7) | c;
+    return 4;
+  }
+  x64 = (x<<7) | (c&0x7f);
+  n = 4;
+  do{
+    c = p[n++];
+    if( n==9 ){
+      x64 = (x64<<8) | c;
+      break;
+    }
+    x64 = (x64<<7) | (c&0x7f);
+  }while( (c & 0x80)!=0 );
+  *v = x64;
+  return n;
+}
+
+/*
+** Read a 32-bit variable-length integer from memory starting at p[0].
+** Return the number of bytes read.  The value is stored in *v.
+*/
+int sqlite3GetVarint32(const unsigned char *p, u32 *v){
+  u32 x;
+  int n;
+  unsigned char c;
+  if( ((c = p[0]) & 0x80)==0 ){
+    *v = c;
+    return 1;
+  }
+  x = c & 0x7f;
+  if( ((c = p[1]) & 0x80)==0 ){
+    *v = (x<<7) | c;
+    return 2;
+  }
+  x = (x<<7) | (c & 0x7f);
+  n = 2;
+  do{
+    x = (x<<7) | ((c = p[n++])&0x7f);
+  }while( (c & 0x80)!=0 && n<9 );
+  *v = x;
+  return n;
+}
+
+/*
+** Return the number of bytes that will be needed to store the given
+** 64-bit integer.
+*/
+int sqlite3VarintLen(u64 v){
+  int i = 0;
+  do{
+    i++;
+    v >>= 7;
+  }while( v!=0 && i<9 );
+  return i;
+}
+
+/*
+** Translate a single byte of Hex into an integer.
+*/
+static int hexToInt(int h){
+  if( h>='0' && h<='9' ){
+    return h - '0';
+  }else if( h>='a' && h<='f' ){
+    return h - 'a' + 10;
+  }else if( h>='A' && h<='F' ){
+    return h - 'A' + 10;
+  }else{
+    return 0;
+  }
+}
+
+/*
+** Convert a BLOB literal of the form "x'hhhhhh'" into its binary
+** value.  Return a pointer to its binary value.  Space to hold the
+** binary value has been obtained from malloc and must be freed by
+** the calling routine.
+*/
+void *sqlite3HexToBlob(const char *z){
+  char *zBlob;
+  int i;
+  int n = strlen(z);
+  if( n%2 ) return 0;
+
+  zBlob = (char *)sqliteMalloc(n/2);
+  for(i=0; i<n; i+=2){
+    zBlob[i/2] = (hexToInt(z[i])<<4) | hexToInt(z[i+1]);
+  }
+  return zBlob;
+}
